#!/usr/bin/env python3
"""Entry point for recursivist.
This allows the package to be run directly using `python -m recursivist`.
"""
from recursivist.cli import main

if __name__ == "__main__":
    main()
